@@ -29,24 +29,84 @@
 # -
 #
 
+#-------------------------------------------------------------------------------
+# Imports!
+#-------------------------------------------------------------------------------
+
+import os
+import sys
+from optparse import OptionParser
+
+import gtk
+import gettext
+
+#-------------------------------------------------------------------------------
+# Find Install Prefix
+# Note: It's important to do this before we import the Screenlets submodules
+#-------------------------------------------------------------------------------
 try:
 	INSTALL_PREFIX = open("/etc/screenlets/prefix").read()[:-1] 
 except:
 	INSTALL_PREFIX = '/usr'
 
+# translation stuff
+gettext.textdomain('screenlets')
+gettext.bindtextdomain('screenlets', INSTALL_PREFIX +  '/share/locale')
+
+#-------------------------------------------------------------------------------
+# Parse command line options
+# Note: This must be done before we import any submodules
+# TODO: Fix up command line parsing. Right now, the same options are parsed for
+#	both screenlets, the manager, the daemon, and melange.
+#-------------------------------------------------------------------------------
+# Define command line related constants.
+# These are used as the default values for command line options that aren't overriden
+LOG_DISABLED = False				# Disable log
+LOG_LEVEL = 4						# TODO: change to 3 for stable version to show only warning, error and critical messages
+LOG_OUTPUT = "FILE"					# default output, allowed FILE, STDERR, STDOUT
+LOG_NAME = "screenlets"		# Log name
+LOG_FILE = "/tmp/%s.log" % os.path.basename(sys.argv[0]) # full path to log file; only used if LOG_OUTPUT is FILE
+
+SESSION_NAME = "default"
+SESSION_REPLACE = "False"
+SERVER_NAME = "none"
+
+# Create the parser
+parser = OptionParser()
+# Add options used by all of the various modules
+parser.add_option("-l", "--logging-level", dest = "LOG_LEVEL", default = LOG_LEVEL,
+	help = ("set logging level.\n0 - log disabled, 1 - only critical errors, 2 - all errors,\
+	3 - all errors and warnings, 4 - all errors, warnings, and info messages, 5 - all messages \
+	including debug output.\ndefault: %s") %(LOG_LEVEL))
+parser.add_option("-f", "--logging-file", dest = "LOG_FILE", default = LOG_FILE,
+	help = ("write log to LOG_FILE. Default: %s") %(LOG_FILE))
+parser.add_option("-s", "--server", dest = "SERVER_NAME", default = SERVER_NAME,
+	help = "server name. default options are Melange and Sidebar")
+parser.add_option("-o", "--output", dest = "LOG_OUTPUT", default = LOG_OUTPUT,
+	help = ("set output. allowed outputs: FILE, STDOUT and STDERR. Default: %s") %(LOG_OUTPUT))
+parser.add_option("-q", "--quiet", action="store_true", dest = "LOG_DISABLED",
+	default = LOG_DISABLED, help = "Disable log. The same as log level 0")
+parser.add_option("-r", "--replace", action="store_true", dest = "SESSION_REPLACE",
+	default = SESSION_REPLACE, help = "Replace old session. Default: %s" %(SESSION_REPLACE))
+parser.add_option("-e", "--session", action="store_true", dest = "SESSION_NAME",
+	default = SESSION_NAME, help = "Name of session. Default: %s" %(SESSION_NAME))
+# Parse the options and store them in global module variables
+COMMAND_LINE_OPTIONS, COMMAND_LINE_ARGS = parser.parse_args()
+
+#-------------------------------------------------------------------------------
+# Finish with the imports and import all of the submodules
+#-------------------------------------------------------------------------------
+
 import pygtk
 pygtk.require('2.0')
-import gtk
 import cairo, pango
 import gobject
 import glib
 try:
 	import rsvg
 except ImportError: print 'No module RSVG , graphics will not be so good'
-import os
 import subprocess
 import glob
-import gettext
 import math
 
 # import screenlet-submodules
@@ -60,9 +120,7 @@ from menu import DefaultMenuItem, add_menuitem
 from drawing import Drawing
 # /TEST
 
-# translation stuff
-gettext.textdomain('screenlets')
-gettext.bindtextdomain('screenlets', INSTALL_PREFIX +  '/share/locale')
+import logger
 
 def _(s):
 	return gettext.gettext(s)
@@ -75,7 +133,7 @@ def _(s):
 APP_NAME = "Screenlets"
 
 # the version of the Screenlets-baseclass in use
-VERSION = "0.1.3"
+VERSION = "0.1.4"
 
 # the application copyright
 COPYRIGHT = "(c) RYX (Rico Pfaus) <ryx@ryxperience.com>\nWhise (Helder Fraga) <helder.fraga@hotmail.com>"
@@ -136,15 +194,15 @@ DEBUG_MODE		= True
 
 DEBIAN = True
 try:
-    subprocess.call(["dpkg"], stdout=open(os.devnull, 'w'), stderr=subprocess.STDOUT)
+	subprocess.call(["dpkg"], stdout=open(os.devnull, 'w'), stderr=subprocess.STDOUT)
 except OSError:
-    DEBIAN = False
+	DEBIAN = False
 
 UBUNTU = True
 try:
-    subprocess.call(["apt-add-repository"], stdout=open(os.devnull, 'w'), stderr=subprocess.STDOUT)
+	subprocess.call(["apt-add-repository"], stdout=open(os.devnull, 'w'), stderr=subprocess.STDOUT)
 except OSError:
-    UBUNTU = False
+	UBUNTU = False
 
 #-------------------------------------------------------------------------------
 # CLASSES
@@ -318,8 +376,8 @@ class ScreenletTheme (dict):
 		"""@DEPRECATED Moved to Screenlets class: Draws a circule"""
 		ctx.save()
 		ctx.translate(x, y)
-       		ctx.arc(width/2,height/2,min(height,width)/2,0,2*math.pi)
-		if fill:ctx.fill()
+		ctx.arc(width/2,height/2,min(height,width)/2,0,2*math.pi)
+		if fill: ctx.fill()
 		else: ctx.stroke()
 		ctx.restore()
 
@@ -328,7 +386,7 @@ class ScreenletTheme (dict):
 		ctx.save()
 		ctx.move_to(start_x, start_y)
 		ctx.set_line_width(line_width)
-        	ctx.rel_line_to(end_x, end_y)
+		ctx.rel_line_to(end_x, end_y)
 		if close : ctx.close_path()
 		if preserve: ctx.stroke_preserve()
 		else: ctx.stroke()
@@ -348,30 +406,30 @@ class ScreenletTheme (dict):
 		ctx.save()
 		ctx.translate(x, y)
 		padding=0 # Padding from the edges of the window
-        	rounded=rounded_angle # How round to make the edges 20 is ok
-        	w = width
+		rounded=rounded_angle # How round to make the edges 20 is ok
+		w = width
 		h = height
 
-        	# Move to top corner
-        	ctx.move_to(0+padding+rounded, 0+padding)
-        	
-        	# Top right corner and round the edge
-        	ctx.line_to(w-padding-rounded, 0+padding)
-        	ctx.arc(w-padding-rounded, 0+padding+rounded, rounded, (math.pi/2 )+(math.pi) , 0)
+		# Move to top corner
+		ctx.move_to(0+padding+rounded, 0+padding)
+			
+		# Top right corner and round the edge
+		ctx.line_to(w-padding-rounded, 0+padding)
+		ctx.arc(w-padding-rounded, 0+padding+rounded, rounded, (math.pi/2 )+(math.pi) , 0)
 	
-        	# Bottom right corner and round the edge
-        	ctx.line_to(w-padding, h-padding-rounded)
-        	ctx.arc(w-padding-rounded, h-padding-rounded, rounded, 0, math.pi/2)
-       	
-        	# Bottom left corner and round the edge.
-        	ctx.line_to(0+padding+rounded, h-padding)
-        	ctx.arc(0+padding+rounded, h-padding-rounded, rounded,math.pi/2, math.pi)
+		# Bottom right corner and round the edge
+		ctx.line_to(w-padding, h-padding-rounded)
+		ctx.arc(w-padding-rounded, h-padding-rounded, rounded, 0, math.pi/2)
+	   	
+		# Bottom left corner and round the edge.
+		ctx.line_to(0+padding+rounded, h-padding)
+		ctx.arc(0+padding+rounded, h-padding-rounded, rounded,math.pi/2, math.pi)
 	
-        	# Top left corner and round the edge
-        	ctx.line_to(0+padding, 0+padding+rounded)
-        	ctx.arc(0+padding+rounded, 0+padding+rounded, rounded, math.pi, (math.pi/2 )+(math.pi))
-        	
-        	# Fill in the shape.
+		# Top left corner and round the edge
+		ctx.line_to(0+padding, 0+padding+rounded)
+		ctx.arc(0+padding+rounded, 0+padding+rounded, rounded, math.pi, (math.pi/2 )+(math.pi))
+			
+		# Fill in the shape.
 		if fill:ctx.fill()
 		else: ctx.stroke()
 		ctx.restore()
@@ -430,29 +488,29 @@ class ScreenletTheme (dict):
 		ctx.restore()
 
 	def show_notification (self,text):
-	        """@DEPRECATED Moved to Screenlets class: Show notification window at current mouse position."""
+		"""@DEPRECATED Moved to Screenlets class: Show notification window at current mouse position."""
 		if self.notify == None:
-	      		self.notify = Notify()
-	        	self.notify.text = text
-	        	self.notify.show()
+		 	self.notify = Notify()
+			self.notify.text = text
+		self.notify.show()
 
 	def hide_notification (self):
-	        """@DEPRECATED Moved to Screenlets class: hide notification window"""
+		"""@DEPRECATED Moved to Screenlets class: hide notification window"""
 		if self.notify != None:
 			self.notify.hide()
 			self.notify = None
 
 	def show_tooltip (self,text,tooltipx,tooltipy):
-	        """@DEPRECATED: Moved to Screenlets class: Show tooltip window at current mouse position."""
+		"""@DEPRECATED: Moved to Screenlets class: Show tooltip window at current mouse position."""
 		if self.tooltip == None:
-      			self.tooltip = Tooltip(300, 400)
-        		self.tooltip.text = text
-        		self.tooltip.x    = tooltipx
-        		self.tooltip.y    = tooltipy
-			self.tooltip.show()
+	  		self.tooltip = Tooltip(300, 400)
+			self.tooltip.text = text
+			self.tooltip.x	= tooltipx
+			self.tooltip.y	= tooltipy
+		self.tooltip.show()
 
 	def hide_tooltip (self):
-	        """@DEPRECATED Moved to Screenlets class: hide tooltip window"""
+		"""@DEPRECATED Moved to Screenlets class: hide tooltip window"""
 		if self.tooltip != None:
 			self.tooltip.hide()
 			self.tooltip = None		
@@ -644,8 +702,8 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 	p_layout		= None		# PangoLayout
 	
 	# default editable options, available for all Screenlets
-	x = 0
-	y = 0
+	x = 200
+	y = 150
 	mousex = 0
 	mousey = 0
 	mouse_is_over = False
@@ -657,8 +715,8 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 	is_visible		= True
 	is_sticky		= False
 	is_widget		= False
-	keep_above		= True
-	keep_below		= False
+	keep_above		= False
+	keep_below		= True
 	skip_pager		= True
 	first_run		= False
 	skip_taskbar	= True
@@ -720,8 +778,8 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 		self.__dict__['is_sticky'] 	= is_sticky
 		self.__dict__['draw_buttons'] 	= draw_buttons
 		self.resize_on_scroll = resize_on_scroll
-		self.__dict__['x'] = 0
-		self.__dict__['y'] = 0
+		self.__dict__['x'] = 200
+		self.__dict__['y'] = 150
 		# TEST: set scale relative to theme size (NOT WORKING)
 		#self.__dict__['scale'] = width/100.0
 		# /TEST
@@ -742,70 +800,80 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 		else: self.draw_buttons = False
 		if uses_theme:
 			self.uses_theme = True
-			self.add_option(StringOption('Screenlet', 'theme_name', 
-				'default', '', '', hidden=True))
+			self.add_option(StringOption('Screenlet', 'theme_name',
+				default='default', hidden=True))
 		# create/add options
-		self.add_option(IntOption('Screenlet', 'x', 
-			0, _('X-Position'), _('The X-position of this Screenlet ...'), 
+		self.add_option(IntOption('Screenlet', 'x',
+			default=200, label=_('X-Position'),
+			desc=_('The X-position of this Screenlet ...'),
 			min=0, max=gtk.gdk.screen_width()))
-		self.add_option(IntOption('Screenlet', 'y', 
-			0, _('Y-Position'), _('The Y-position of this Screenlet ...'), 
+		self.add_option(IntOption('Screenlet', 'y',
+			default=150, label=_('Y-Position'),
+			desc=_('The Y-position of this Screenlet ...'),
 			min=0, max=gtk.gdk.screen_height()))
-		self.add_option(IntOption('Screenlet', 'width', 
-			width, _('Width'), _('The width of this Screenlet ...'), 
+		self.add_option(IntOption('Screenlet', 'width',
+			default=width, label=_('Width'),
+			desc=_('The width of this Screenlet ...'),
 			min=16, max=1000, hidden=True))
-		self.add_option(IntOption('Screenlet', 'height', 
-			height, _('Height'), _('The height of this Screenlet ...'), 
+		self.add_option(IntOption('Screenlet', 'height',
+			default=height, label=_('Height'),
+			desc=_('The height of this Screenlet ...'),
 			min=16, max=1000, hidden=True))
-		self.add_option(FloatOption('Screenlet', 'scale', 
-			self.scale, _('Scale'), _('The scale-factor of this Screenlet ...'), 
+		self.add_option(FloatOption('Screenlet', 'scale',
+			default=self.scale, label=_('Scale'),
+			desc=_('The scale-factor of this Screenlet ...'),
 			min=0.1, max=10.0, digits=2, increment=0.1))
-		self.add_option(FloatOption('Screenlet', 'opacity', 
-			self.opacity, _('Opacity'), _('The opacity of the Screenlet window ...'), 
+		self.add_option(FloatOption('Screenlet', 'opacity',
+			default=self.opacity, label=_('Opacity'),
+			desc=_('The opacity of the Screenlet window ...'),
 			min=0.1, max=1.0, digits=2, increment=0.1))
-		self.add_option(BoolOption('Screenlet', 'is_sticky', 
-			is_sticky, _('Stick to Desktop'), 
-			_('Show this Screenlet on all workspaces ...')))
-		self.add_option(BoolOption('Screenlet', 'is_widget', 
-			is_widget, _('Treat as Widget'), 
-			_('Treat this Screenlet as a "Widget" ...')))
-		self.add_option(BoolOption('Screenlet', 'is_dragged', 
-			self.is_dragged, "Is the screenlet dragged","Is the screenlet dragged", hidden=True))
-		self.add_option(BoolOption('Screenlet', 'is_sizable', 
-			is_sizable, "Can the screenlet be resized","is_sizable", hidden=True))
-		self.add_option(BoolOption('Screenlet', 'is_visible', 
-			self.is_visible, "Usefull to use screenlets as gnome panel applets","is_visible", hidden=True))
-		self.add_option(BoolOption('Screenlet', 'lock_position', 
-			self.lock_position, _('Lock position'), 
-			_('Stop the screenlet from being moved...')))
-		self.add_option(BoolOption('Screenlet', 'keep_above', 
-			self.keep_above, _('Keep above'), 
-			_('Keep this Screenlet above other windows ...')))
-		self.add_option(BoolOption('Screenlet', 'keep_below', 
-			self.keep_below, _('Keep below'), 
-			_('Keep this Screenlet below other windows ...')))
-		self.add_option(BoolOption('Screenlet', 'draw_buttons', 
-			self.draw_buttons, _('Draw button controls'), 
-			_('Draw buttons in top right corner')))
-		self.add_option(BoolOption('Screenlet', 'skip_pager', 
-			self.skip_pager, _('Skip Pager'), 
-			_('Set this Screenlet to show/hide in pagers ...')))
-		self.add_option(BoolOption('Screenlet', 'skip_taskbar', 
-			self.skip_pager, _('Skip Taskbar'), 
-			_('Set this Screenlet to show/hide in taskbars ...')))
-		self.add_option(BoolOption('Screenlet', 'resize_on_scroll', 
-			self.resize_on_scroll, _("Resize on mouse scroll"),"resize_on_scroll"))
+		self.add_option(BoolOption('Screenlet', 'is_sticky',
+			default=is_sticky, label=_('Stick to Desktop'),
+			desc=_('Show this Screenlet on all workspaces ...')))
+		self.add_option(BoolOption('Screenlet', 'is_widget',
+			default=is_widget, label=_('Treat as Widget'),
+			desc=_('Treat this Screenlet as a "Widget" ...')))
+		self.add_option(BoolOption('Screenlet', 'is_dragged',
+			default=self.is_dragged, label="Is the screenlet dragged",
+			desc="Is the screenlet dragged", hidden=True))
+		self.add_option(BoolOption('Screenlet', 'is_sizable',
+			default=is_sizable, label="Can the screenlet be resized",
+			desc="is_sizable", hidden=True))
+		self.add_option(BoolOption('Screenlet', 'is_visible',
+			default=self.is_visible, label="Usefull to use screenlets as gnome panel applets",
+			desc="is_visible", hidden=True))
+		self.add_option(BoolOption('Screenlet', 'lock_position',
+			default=self.lock_position, label=_('Lock position'),
+			desc=_('Stop the screenlet from being moved...')))
+		self.add_option(BoolOption('Screenlet', 'keep_above',
+			default=self.keep_above, label=_('Keep above'),
+			desc=_('Keep this Screenlet above other windows ...')))
+		self.add_option(BoolOption('Screenlet', 'keep_below',
+			default=self.keep_below, label=_('Keep below'),
+			desc=_('Keep this Screenlet below other windows ...')))
+		self.add_option(BoolOption('Screenlet', 'draw_buttons',
+			default=self.draw_buttons, label=_('Draw button controls'),
+			desc=_('Draw buttons in top right corner')))
+		self.add_option(BoolOption('Screenlet', 'skip_pager',
+			default=self.skip_pager, label=_('Skip Pager'),
+			desc=_('Set this Screenlet to show/hide in pagers ...')))
+		self.add_option(BoolOption('Screenlet', 'skip_taskbar',
+			default=self.skip_pager, label=_('Skip Taskbar'),
+			desc=_('Set this Screenlet to show/hide in taskbars ...')))
+		self.add_option(BoolOption('Screenlet', 'resize_on_scroll',
+			default=self.resize_on_scroll, label=_("Resize on mouse scroll"),
+			desc="resize_on_scroll"))
 		self.add_option(BoolOption('Screenlet', 'ignore_requirements', 
 			self.ignore_requirements, _('Ignore requirements'), 
 			_('Set this Screenlet to ignore/demand DEB requirements ...')))
 		if uses_theme:
 			self.ask_on_option_override = ask_on_option_override
-			self.add_option(BoolOption('Screenlet', 'allow_option_override', 
-			self.allow_option_override, _('Allow overriding Options'), 
-				_('Allow themes to override options in this screenlet ...')))
-			self.add_option(BoolOption('Screenlet', 'ask_on_option_override', 
-				self.ask_on_option_override, _('Ask on Override'), 
-				_('Show a confirmation-dialog when a theme wants to override ')+\
+			self.add_option(BoolOption('Screenlet', 'allow_option_override',
+				default=self.allow_option_override, label=_('Allow overriding Options'),
+				desc=_('Allow themes to override options in this screenlet ...')))
+			self.add_option(BoolOption('Screenlet', 'ask_on_option_override',
+				default=self.ask_on_option_override, label=_('Ask on Override'),
+				desc=_('Show a confirmation-dialog when a theme wants to override ')+\
 				_('the current options of this Screenlet ...')))
 		# disable width/height
 		self.disable_option('width')
@@ -818,6 +886,10 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 			self.window.set_destroy_with_parent(True)
 		self.window.resize(width, height)
 		self.window.set_decorated(False)
+		try:	# Workaround for Ubuntu Natty
+			self.window.set_property('has-resize-grip', False)
+		except TypeError:
+			pass
 		self.window.set_app_paintable(True)
 		# create pango layout, if active
 		if uses_pango:
@@ -835,7 +907,8 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 			print "WARNING - You are using kwin window manager , screenlets doesnt have full compatibility with this window manager"
 		else:
 			self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_TOOLBAR)
-		self.window.set_keep_above(True)
+		self.window.set_keep_above(self.keep_above)
+		self.window.set_keep_below(self.keep_below)
 		self.window.set_skip_taskbar_hint(True)
 		self.window.set_skip_pager_hint(True)
 		if is_sticky:
@@ -883,8 +956,11 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 			
 		if show_window:
 			self.window.show()
-#			print os.environ['HOME'] + '/.config/Screenlets/' + self.__name__[:-9] + '/default/'+ self.id
-			if not os.path.exists(os.environ['HOME'] + '/.config/Screenlets/' + self.__name__[:-9] + '/default/'+ self.id + '.ini'):
+			if self.__name__.endswith("Screenlet"):
+				short_name = self.__name__[:-9]
+			else:
+				short_name = self.__name__
+			if not os.path.exists(os.environ['HOME'] + '/.config/Screenlets/' + short_name + '/default/'+ self.id + '.ini'):
 				self.first_run = True
 			self.window.hide()	
 
@@ -959,7 +1035,7 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 			if self.window.window:
 				self.window.window.set_skip_taskbar_hint(bool(value))
 		# NOTE: This is the new recommended way of storing options in real-time
-		#       (we access the backend through the session here)
+		#	   (we access the backend through the session here)
 		if self.saving_enabled:
 			o = self.get_option_by_name(name)
 			if o != None:
@@ -1511,14 +1587,14 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 		if self.window.is_composited():
 #			log.debug(_("Updating input shape"))
 			self.on_draw_shape(ctx)
-			self.main_view.set_shape(self.__shape_bitmap, True)
+#			self.main_view.set_shape(self.__shape_bitmap, True)
 		else:
 			try:
 				self.on_draw_shape(ctx)
 			except:
 				self.on_draw(ctx)
 #			log.debug(_("Updating window shape"))
-			self.main_view.set_shape(self.__shape_bitmap, False)
+#			self.main_view.set_shape(self.__shape_bitmap, False)
 
 	def update_shape (self):
 		"""Update window shape (only call this when shape has changed
@@ -1929,13 +2005,13 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 	
 	def expose (self, widget, event):
 		ctx = widget.window.cairo_create()
-		# clear context
-		self.clear_cairo_context(ctx)
 		# set a clip region for the expose event
 		ctx.rectangle(event.area.x, event.area.y,
 			event.area.width, event.area.height)
 		ctx.clip()
-		
+		# clear context
+		self.clear_cairo_context(ctx)
+	
 		# scale context
 		#ctx.scale(self.scale, self.scale)
 		# call drawing method
@@ -2084,35 +2160,35 @@ class Screenlet (gobject.GObject, EditableOptions, Drawing):
 
 
 	def show_notification (self,text):
-	        """Show notification window at current mouse position."""
+		"""Show notification window at current mouse position."""
 		if self.notify == None:
-	      		self.notify = Notify()
-	        	self.notify.text = text
-	        	self.notify.show()
+			self.notify = Notify()
+			self.notify.text = text
+			self.notify.show()
 
 	def hide_notification (self):
-	        """hide notification window"""
+		"""hide notification window"""
 		if self.notify != None:
 			self.notify.hide()
 			self.notify = None
 
 	def show_tooltip (self,text,tooltipx,tooltipy):
-	        """Show tooltip window at current mouse position."""
+		"""Show tooltip window at current mouse position."""
 		if self.tooltip == None:
-      			self.tooltip = Tooltip(300, 400)
-        		self.tooltip.text = text
-        		self.tooltip.x    = tooltipx
-        		self.tooltip.y    = tooltipy
+	  		self.tooltip = Tooltip(300, 400)
+			self.tooltip.text = text
+			self.tooltip.x	= tooltipx
+			self.tooltip.y	= tooltipy
 			self.tooltip.show()
 		else:
-     			#self.tooltip = Tooltip(300, 400)
-        		self.tooltip.text = text
-        		self.tooltip.x    = tooltipx
-        		self.tooltip.y    = tooltipy
+	 		#self.tooltip = Tooltip(300, 400)
+			self.tooltip.text = text
+			self.tooltip.x	= tooltipx
+			self.tooltip.y	= tooltipy
 			#self.tooltip.show()
 
 	def hide_tooltip (self):
-	        """hide tooltip window"""
+		"""hide tooltip window"""
 		if self.tooltip != None:
 			self.tooltip.hide()
 			self.tooltip = None		
@@ -2200,25 +2276,67 @@ class ShapedWidget (gtk.DrawingArea):
 		del ctx
 		return False
 
+class WrapLabel(gtk.Label):
+    __gtype_name__ = 'WrapLabel'
+
+    def __init__(self, str=None):
+        gtk.Label.__init__(self)
+        
+        self.__wrap_width = 0
+        self.layout = self.get_layout()
+        self.layout.set_wrap(pango.WRAP_WORD_CHAR)
+        
+        if str != None:
+            self.set_text(str)
+        
+        self.set_alignment(0.0, 0.0)
+    
+    def do_size_request(self, requisition):
+        layout = self.get_layout()
+        width, height = layout.get_pixel_size()
+        requisition.width = 0
+        requisition.height = height
+    
+    def do_size_allocate(self, allocation):
+        gtk.Label.do_size_allocate(self, allocation)
+        self.__set_wrap_width(allocation.width)
+    
+    def set_text(self, str):
+        gtk.Label.set_text(self, str)
+        self.__set_wrap_width(self.__wrap_width)
+        
+    def set_markup(self, str):
+        gtk.Label.set_markup(self, str)
+        self.__set_wrap_width(self.__wrap_width)
+    
+    def __set_wrap_width(self, width):
+        if width == 0:
+            return
+        layout = self.get_layout()
+        layout.set_width(width * pango.SCALE)
+        if self.__wrap_width != width:
+            self.__wrap_width = width
+            self.queue_resize()
+
 class Tooltip(object):
 	"""A window that displays a text and serves as Tooltip (very basic yet)."""
 	
 	# internals
-	__timeout    = None
-    
+	__timeout	= None
+	
 	# attribs
-	text        = ''
-	font_name    = 'FreeSans 9'
-	width        = 100
-	height        = 20
-	x             = 0
-	y             = 0
-    
+	text		= ''
+	font_name	= 'FreeSans 9'
+	width		= 100
+	height		= 20
+	x			 = 0
+	y			 = 0
+	
 	def __init__ (self, width, height):
 		object.__init__(self)
 		# init
-		self.__dict__['width']    = width
-		self.__dict__['height']    = height
+		self.__dict__['width']	= width
+		self.__dict__['height']	= height
 		self.window = gtk.Window()
 		self.window.set_app_paintable(True)
 		self.window.set_size_request(width, height)
@@ -2228,23 +2346,32 @@ class Tooltip(object):
 		self.window.set_skip_taskbar_hint(True)
 		self.window.set_keep_above(True)
 		self.screen_changed(self.window)
-		self.window.connect("expose_event", self.expose)
 		self.window.connect("screen-changed", self.screen_changed)
 		#self.window.show()
-		self.p_context = self.window.get_pango_context()
-		self.p_layout = pango.Layout(self.p_context)
-		self.p_layout.set_font_description(\
-		pango.FontDescription(self.font_name))
-		#self.p_layout.set_width(-1)
+
+		try:    # Workaround for Ubuntu Natty
+			self.window.set_property('has-resize-grip', False)
+		except TypeError:
+			pass
+		self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(65535, 65535, 32767))
+		self.label = WrapLabel()
+		self.label.modify_font(pango.FontDescription(self.font_name))
+		self.label.set_line_wrap(True)
+		self.label.show()
+		self.window.add(self.label)
+
+		self.p_layout = self.label.get_layout()
 		self.p_layout.set_width(width * pango.SCALE - 6)
-    
+	
 	def __setattr__ (self, name, value):
 		self.__dict__[name] = value
 		if name in ('width', 'height', 'text'):
 			if name== 'width':
 				self.p_layout.set_width(width)
 			elif name == 'text':
+				value = utils.html_to_pango(value)
 				self.p_layout.set_markup(value)
+				self.label.set_markup(value)
 				ink_rect, logical_rect = self.p_layout.get_pixel_extents()
 				self.height = min(max(logical_rect[3], 16), 400) + 6
 				self.window.set_size_request(self.width, self.height)
@@ -2253,7 +2380,7 @@ class Tooltip(object):
 			self.window.move(int(value), int(self.y))
 		elif name == 'y':
 			self.window.move(int(self.x), int(value))
-    
+	
 	def show (self):
 		"""Show the Tooltip window."""
 		self.cancel_show()
@@ -2264,22 +2391,22 @@ class Tooltip(object):
 		"""Show the Tooltip window after a given delay."""
 		self.cancel_show()
 		self.__timeout = gobject.timeout_add(delay, self.__show_timeout)
-    
+	
 	def hide (self):
 		"""Hide the Tooltip window."""
 		self.cancel_show()
 		self.window.destroy()
-    
+	
 	def cancel_show (self):
 		"""Cancel showing of the Tooltip."""
 		if self.__timeout:
 			gobject.source_remove(self.__timeout)
 			self.p_context = None
 			self.p_layout = None
-    
+	
 	def __show_timeout (self):
 		self.show()
-    
+	
 	def screen_changed (self, window, screen=None):
 		if screen == None:
 			screen = window.get_screen()
@@ -2287,47 +2414,22 @@ class Tooltip(object):
 		if not map:
 			map = screen.get_rgb_colormap()
 		window.set_colormap(map)
-    
-	def expose (self, widget, event):
-		ctx = self.window.window.cairo_create()
-		ctx.set_antialias (cairo.ANTIALIAS_SUBPIXEL)    # ?
-		# set a clip region for the expose event
-		ctx.rectangle(event.area.x, event.area.y,event.area.width, event.area.height)
-		ctx.clip()
-		# clear context
-		ctx.set_source_rgba(1, 1, 1, 0)
-		ctx.set_operator (cairo.OPERATOR_SOURCE)
-		ctx.paint()
-		# draw rectangle
-		ctx.set_source_rgba(1, 1, 0.5, 1)
-		ctx.rectangle(0, 0, self.width, self.height)
-		ctx.fill()
-		# draw text
-		ctx.save()
-		ctx.translate(3, 3)
-		ctx.set_source_rgba(0, 0, 0, 1) 
-		ctx.show_layout(self.p_layout)
-		ctx.fill()
-		ctx.restore()
-		ctx.rectangle(0, 0, self.width, self.height)
-		ctx.set_source_rgba(0, 0, 0, 0.7)
-		ctx.stroke()
 
 class Notify(object):
 	"""A window that displays a text and serves as Notification (very basic yet)."""
 	
 	# internals
-	__timeout    = None
-    
+	__timeout	= None
+	
 	# attribs
-	text        = ''
-	font_name    = 'FreeSans 9'
-	width        = 200
-	height        = 100
-	x             = 0
-	y             = 0
+	text		= ''
+	font_name	= 'FreeSans 9'
+	width		= 200
+	height		= 100
+	x			 = 0
+	y			 = 0
 	gradient = cairo.LinearGradient(0, 100,0, 0)
-    
+	
 	def __init__ (self):
 		object.__init__(self)
 		# init
@@ -2349,7 +2451,7 @@ class Notify(object):
 		pango.FontDescription(self.font_name))
 		#self.p_layout.set_width(-1)
 		self.p_layout.set_width(self.width * pango.SCALE - 6)
-    
+	
 	def __setattr__ (self, name, value):
 		self.__dict__[name] = value
 		if name in ('text'):
@@ -2369,22 +2471,22 @@ class Notify(object):
 		"""Show the Notify window after a given delay."""
 		self.cancel_show()
 		self.__timeout = gobject.timeout_add(delay, self.__show_timeout)
-    
+	
 	def hide (self):
 		"""Hide the Notify window."""
 		self.cancel_show()
 		self.window.destroy()
-    
+	
 	def cancel_show (self):
 		"""Cancel showing of the Notify."""
 		if self.__timeout:
 			gobject.source_remove(self.__timeout)
 			self.p_context = None
 			self.p_layout = None
-    
+	
 	def __show_timeout (self):
 		self.show()
-    
+	
 	def screen_changed (self, window, screen=None):
 		if screen == None:
 			screen = window.get_screen()
@@ -2392,10 +2494,10 @@ class Notify(object):
 		if not map:
 			map = screen.get_rgb_colormap()
 		window.set_colormap(map)
-    
+	
 	def expose (self, widget, event):
 		ctx = self.window.window.cairo_create()
-		ctx.set_antialias (cairo.ANTIALIAS_SUBPIXEL)    # ?
+		ctx.set_antialias (cairo.ANTIALIAS_SUBPIXEL)	# ?
 		# set a clip region for the expose event
 		ctx.rectangle(event.area.x, event.area.y,event.area.width, event.area.height)
 		ctx.clip()
