@@ -25,6 +25,8 @@ gettext.bindtextdomain('screenlets', screenlets.INSTALL_PREFIX +  '/share/locale
 import gobject
 from distutils.version import LooseVersion
 from subprocess import *
+from HTMLParser import HTMLParser
+from BeautifulSoup import BeautifulStoneSoup
 try:
 	import gnomevfs
 except:
@@ -37,6 +39,25 @@ def _(s):
 # ------------------------------------------------------------------------------
 
 
+class MLStripper(HTMLParser):
+    def __init__(self):
+        self.reset()
+        self.fed = []
+    def handle_data(self, d):
+        self.fed.append(d)
+    def get_data(self):
+        return ''.join(self.fed)
+
+def html_to_pango (html):	
+	"""Simple html to pango stripper."""
+	s = MLStripper()
+	s.feed(html)
+	no_html = s.get_data()
+	decoded = BeautifulStoneSoup(no_html, convertEntities=BeautifulStoneSoup.HTML_ENTITIES)
+	result = decoded.encode("UTF-8")
+	return result.strip(" \n")
+
+
 def get_autostart_dir():
 	"""Returns the system autostart directory"""
         desktop_environment = 'gnome'
