@@ -526,7 +526,8 @@ class ScreenletsManager(object):
 			f = open(DIR_USER + '/config.ini', 'w')
 			f.write("[Options]\n")
 			f.write("show_in_tray=True\n")
-			f.write("Keep_above=True\n")
+			f.write("Keep_above=False\n")
+			f.write("Keep_below=True\n")
 			f.close()
 		try:
 			if ini.load(DIR_USER + '/config.ini'):
@@ -540,7 +541,8 @@ class ScreenletsManager(object):
 			f = open(DIR_USER + '/config.ini', 'w')
 			f.write("[Options]\n")
 			f.write("show_in_tray=True\n")
-			f.write("Keep_above=True\n")
+			f.write("Keep_above=False\n")
+			f.write("Keep_below=True\n")
 			f.close()
 			if ini.load(DIR_USER + '/config.ini'):
 				show_in_tray = ini.get_option('show_in_tray', section='Options')
@@ -1058,7 +1060,6 @@ class ScreenletsManager(object):
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/WebappScreenlet.py ' +DIR_USER + '/' + a + '/' + a + 'Screenlet.py')
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/webapp.png ' +DIR_USER + '/' + a + '/icon.png')				
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/webapp.png ' +DIR_USER + '/' + a + '/themes/default')
-						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/prefs.js ' +DIR_USER + '/' + a + '/mozilla')			
 		
 						enginecopy = open(DIR_USER + '/' + a + '/' + a + 'Screenlet.py','r')
 						enginesave = enginecopy.read()
@@ -1145,7 +1146,6 @@ class ScreenletsManager(object):
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/WidgetScreenlet.py ' +DIR_USER + '/' + a + '/' + a + 'Screenlet.py')
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/widget.png ' +DIR_USER + '/' + a + '/icon.png')				
 						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/widget.png ' +DIR_USER + '/' + a + '/themes/default')
-						os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/screenlets-manager/prefs.js ' +DIR_USER + '/' + a + '/mozilla')			
 						enginecopy = open(DIR_USER + '/' + a + '/' + a + 'Screenlet.py','r')
 						enginesave = enginecopy.read()
 						enginesave = enginesave.replace('WidgetScreenlet',a + 'Screenlet')
